import { memo, useRef, useState } from 'react';
import { Handle, Position } from 'reactflow';
import { SchemaReactflowRelationNode, toHandleId } from '../../../model';
import styles from './relationpill.module.scss';

/**
 * Component to render a relation flow element
 * @param { FlowElement<RelationData>} param0 The data of a relation flow element.
 */
export const RelationPill = memo((node: SchemaReactflowRelationNode) => {
  const data = node.data;
  const minRef = useRef<HTMLInputElement>(null);
  const maxRef = useRef<HTMLInputElement>(null);

  const [readOnlyMin, setReadOnlyMin] = useState(true);
  const [readOnlyMax, setReadOnlyMax] = useState(true);

  const onDepthChanged = (depth: string) => {
    // Don't allow depth above 99
    const limit = 99;
    if (data?.depth != undefined) {
      data.depth.min = data.depth.min >= limit ? limit : data.depth.min;
      data.depth.max = data.depth.max >= limit ? limit : data.depth.max;

      // Check for for valid depth: min <= max
      if (depth == 'min') {
        if (data.depth.min > data.depth.max) data.depth.max = data.depth.min;
        setReadOnlyMin(true);
      } else if (depth == 'max') {
        if (data.depth.max < data.depth.min) data.depth.min = data.depth.max;
        setReadOnlyMax(true);
      }

      // Set to the correct width
      if (maxRef.current) maxRef.current.style.maxWidth = calcWidth(data.depth.max);
      if (minRef.current) minRef.current.style.maxWidth = calcWidth(data.depth.min);
    }
  };

  const isNumber = (x: string) => {
    {
      if (typeof x != 'string') return false;
      return !Number.isNaN(x) && !Number.isNaN(parseFloat(x));
    }
  };

  const calcWidth = (data: number) => {
    return data.toString().length + 0.5 + 'ch';
  };

  return (
    <div className={styles.relation}>
      <div className={styles.relationWrapper}>
        <span className={styles.relationHandleFiller}>
          {data.leftEntityHandleId && (
            <Handle id={toHandleId(data.leftEntityHandleId)} type="target" position={Position.Left} className={styles.relationHandleLeft} />
          )}
        </span>
        {/* <span className={styles.relationHandleFiller}>
          <Handle
            id={getHandleId(data.name, data.type, Handles.ToAttribute, '')}
            type="target"
            position={Position.Left}
            className={styles.relationHandleAttribute + ' ' + (false ? styles.handleConnectedFill : '')}
          />
        </span>
        <span className={styles.relationHandleFiller}>
          <Handle
            id={getHandleId(data.name, data.type, Handles.ReceiveFunction, '')}
            type="target"
            position={Position.Left}
            className={styles.relationHandleFunction + ' ' + (false ? styles.handleConnectedFill : '')}
          />
        </span> */}
        <div className={styles.relationDataWrapper}>
          <span className={styles.relationSpan}>{data?.name}</span>
          <span className={styles.relationInputHolder}>
            <span>[</span>
            <input
              className={styles.relationInput + ' ' + (readOnlyMin ? styles.relationReadonly : '')}
              ref={minRef}
              type="string"
              min={0}
              readOnly={readOnlyMin}
              placeholder={'?'}
              value={data?.depth.min}
              onChange={(e) => {
                if (data != undefined) {
                  data.depth.min = isNumber(e.target.value) ? parseInt(e.target.value) : 0;
                  e.target.style.maxWidth = calcWidth(data.depth.min);
                }
              }}
              onDoubleClick={() => {
                setReadOnlyMin(false);
              }}
              onBlur={(e) => {
                onDepthChanged('min');
              }}
              onKeyDown={(e) => {
                if (e.key === 'Enter') {
                  onDepthChanged('min');
                }
              }}
            ></input>
            <span>..</span>
            <input
              className={styles.relationInput + ' ' + (readOnlyMax ? styles.relationReadonly : '')}
              ref={maxRef}
              type="string"
              min={0}
              readOnly={readOnlyMax}
              placeholder={'?'}
              value={data?.depth.max}
              onChange={(e) => {
                if (data != undefined) {
                  data.depth.max = isNumber(e.target.value) ? parseInt(e.target.value) : 0;
                  e.target.style.maxWidth = calcWidth(data.depth.max);
                }
              }}
              onDoubleClick={() => {
                setReadOnlyMax(false);
              }}
              onBlur={(e) => {
                onDepthChanged('max');
              }}
              onKeyDown={(e) => {
                if (e.key === 'Enter') {
                  onDepthChanged('max');
                }
              }}
            ></input>
            <span>]</span>
          </span>
        </div>
        <span className={styles.relationHandleFiller}>
          {data.rightEntityHandleId && (
            <Handle
              id={toHandleId(data.rightEntityHandleId)}
              type="source"
              position={Position.Right}
              className={styles.relationHandleRight + ' ' + (false ? styles.handleConnectedBorderRight : '')}
            />
          )}
        </span>
      </div>
    </div>
  );
});

RelationPill.displayName = 'RelationPill';
export default RelationPill;
